"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBsc = void 0;
const ethers_1 = require("ethers");
const utils_1 = require("ethers/lib/utils");
const multicallAddr = "0xcA11bde05977b3631167028862bE2a173976CA11";
const abi = [
    "function balanceOf(address) external returns (uint256)",
    "function getETHBalance(address) external returns (uint256)",
    "function aggregate((address,bytes)[] memory calls) view external returns (uint256 blockNumber, bytes[] memory returnData)",
];
const inf = new utils_1.Interface(abi);
// const Getter = "0x8e82F2b42811054904c702EC1186DB24c6892b2E";
const Getter = "0x6726395AF114D916dF572055f22Ec58F697AA70e";
const ETH_ADDRS = [
    "0xf977814e90da44bfa03b6295a0616a897441acec",
    "0xBE0eB53F46cd790Cd13851d5EFf43D12404d33E8",
    "0x5a52e96bacdabb82fd05763e25335261b270efcb",
    "0x3c783c21a0383057d128bae431894a5c19f9cf06",
    "0xdccf3b77da55107280bd850ea519df3705d1a75a",
    "0x8894e0a0c962cb723c1976a4421c95949be2d4e3",
    "0x515b72ed8a97f42c568d6a143232775018f133c8",
    "0xbd612a3f30dca67bf60a39fd0d35e39b7ab80774",
    "0x01c952174c24e1210d26961d456a77a39e1f0bb0",
    "0x29bdfbf7d27462a2d115748ace2bd71a2646946c",
    "0xe2fc31f816a9b94326492132018c3aecc4a93ae1",
    "0x73f5ebe90f27b46ea12e5795d16c4b408b19cc6f",
    "0x161ba15a5f335c9f06bb5bbb0a9ce14076fbb645",
    "0x1fbe2acee135d991592f167ac371f3dd893a508b",
    "0xeb2d2f1b8c558a40207669291fda468e50c8a0bb",
    "0xa180fe01b906a1be37be6c534a3300785b20d947"
];
const providerUrl = "https://binance.nodereal.io";
function encodeData() {
    let res = [];
    for (let i = 0; i < ETH_ADDRS.length; i++) {
        let addr = ETH_ADDRS[i];
        let calldata = inf.encodeFunctionData("getETHBalance", [addr]);
        res.push([Getter, calldata]);
    }
    return res;
}
function decodeData(returnData) {
    let res = [];
    let offset = 0;
    for (let i = 0; i < ETH_ADDRS.length; i++) {
        let index = i + offset;
        let addr = ETH_ADDRS[i];
        let returndata = returnData[index];
        let amount = inf.decodeFunctionResult("getETHBalance", returndata);
        res.push({
            "account": addr,
            "balance": amount[0].toString(),
            "decimals": 18,
            "symbol": "BNB",
            "chain": "BSC"
        });
    }
    return res;
}
function getBsc() {
    return __awaiter(this, void 0, void 0, function* () {
        let provider = new ethers_1.ethers.providers.JsonRpcProvider(providerUrl);
        const multicall = new ethers_1.ethers.Contract(multicallAddr, abi, provider);
        let data = encodeData();
        let resCall = yield multicall.functions.aggregate(data);
        let blockNumber = resCall.blockNumber;
        let res = decodeData(resCall.returnData);
        console.log(res);
        return res;
    });
}
exports.getBsc = getBsc;
// getBsc()
