"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DbWriter2 = exports.DbWriter = void 0;
const sequelize_1 = require("sequelize");
const types_1 = require("./types");
class DbWriter {
    constructor(connectionInfo, tableName, chain, globalParamsTable) {
        const instance = new sequelize_1.Sequelize(connectionInfo, { "logging": true });
        this.globalPramasTable = globalParamsTable;
        this.instance = instance;
        this.chain = chain;
        this.tableName = tableName;
        this.dbWriter = instance.define('dbWriter', {
            'id': {
                type: sequelize_1.DataTypes.BIGINT,
                allowNull: false,
                autoIncrement: true,
                primaryKey: true
            },
            'pool_address': sequelize_1.DataTypes.CHAR,
            'token_address': sequelize_1.DataTypes.CHAR,
            'chain_id': sequelize_1.DataTypes.INTEGER,
            'balance': sequelize_1.DataTypes.BIGINT,
            'timestamp': sequelize_1.DataTypes.INTEGER,
            'createdAt': sequelize_1.DataTypes.DATE,
            'updatedAt': sequelize_1.DataTypes.DATE
        }, {
            'tableName': tableName,
            'timestamps': false
        });
    }
    getLastReportSumBySymbol(symbol) {
        return __awaiter(this, void 0, void 0, function* () {
            let token_address = (0, types_1.symbolToAddrByChain)(symbol, this.chain);
            let suffix = "_Last_Timestamp";
            let key = `${this.chain}_${symbol}_${suffix}`;
            let sql = `
        SELECT sum(balance) as balance, avg(timestamp) as timestamp
        FROM
            ${this.tableName}
        WHERE
            token_address = '${token_address}'
            and timestamp = IFNULL(
            (
                SELECT valueInt
                FROM
                    ${this.globalPramasTable}
                where
                    keyStr = '${key}'
                ORDER BY timestamp DESC
                LIMIT 1
            ),(
                SELECT TIMESTAMP
                FROM
                    ${this.tableName}
                where token_address = '${token_address}'
                ORDER BY TIMESTAMP 
                LIMIT 1
            ))
        order by timestamp
        limit 1
        `;
            let res = yield this.instance.query(sql, { type: sequelize_1.QueryTypes.SELECT });
            let balance = Number(res[0].balance);
            let timestamp = Number(res[0].timestamp);
            if (timestamp < 1000) {
                console.error("get data from db failed timestamp: %d, balance %d", timestamp, balance);
                process.exit(1);
            }
            let sumInfo = { balance: balance, timestamp: timestamp, chain: this.chain, symbol: symbol };
            console.log("last type", typeof (balance));
            return sumInfo;
        });
    }
    getFirstTodaySumBySymbol(symbol) {
        return __awaiter(this, void 0, void 0, function* () {
            let token_address = (0, types_1.symbolToAddrByChain)(symbol, this.chain);
            let sql = `
        SELECT sum(balance) as balance, avg(timestamp) as timestamp
        FROM
            ${this.tableName}
        WHERE
            token_address = '${token_address}'
            and timestamp = (
                SELECT TIMESTAMP
                FROM
                    ${this.tableName}
                where
                    Date(now()) = Date(FROM_UNIXTIME(timestamp))
                    and token_address = '${token_address}'
                ORDER BY TIMESTAMP
                LIMIT 1
            )
        limit 1
        `;
            let res = yield this.instance.query(sql, { type: sequelize_1.QueryTypes.SELECT });
            let balance = Number(res[0].balance);
            let timestamp = Number(res[0].timestamp);
            if (timestamp < 1000) {
                console.error("get data from db failed timestamp: %d, balance %d", timestamp, balance);
                process.exit(1);
            }
            let sumInfo = { balance: balance, timestamp: timestamp, chain: this.chain, symbol: symbol };
            console.log("last type", typeof (balance));
            return sumInfo;
        });
    }
    getCurrSumBySymbol(symbol) {
        return __awaiter(this, void 0, void 0, function* () {
            let token_address = (0, types_1.symbolToAddrByChain)(symbol, this.chain);
            let sql = `
        SELECT sum(balance) as balance, avg(timestamp) as timestamp
        FROM
            ${this.tableName}
        WHERE
            token_address = '${token_address}'
            and timestamp = (
                SELECT TIMESTAMP
                FROM
                    ${this.tableName}
                where 
                    token_address = '${token_address}'
                ORDER BY TIMESTAMP DESC
                LIMIT 1
            )
        limit 1
        `;
            let res = yield this.instance.query(sql, { type: sequelize_1.QueryTypes.SELECT });
            // console.log(res)
            let balance = Number(res[0].balance);
            let timestamp = Number(res[0].timestamp);
            if (timestamp < 1000) {
                console.error("get data from db failed timestamp: %d, balance %d", timestamp, balance);
                process.exit(1);
            }
            let sumInfo = { balance: balance, timestamp: timestamp, chain: this.chain, symbol: symbol };
            console.log("last type", typeof (balance));
            return sumInfo;
        });
    }
    write(params) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.dbWriter.create(params);
            return res;
        });
    }
    writeAll(params) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.dbWriter.bulkCreate(params);
            return res;
        });
    }
}
exports.DbWriter = DbWriter;
class DbWriter2 {
    constructor(connectionInfo) {
        const instance = new sequelize_1.Sequelize(connectionInfo, { "logging": false });
        this.dbWriter = instance.define('dbWriter', {
            'id': {
                type: sequelize_1.DataTypes.BIGINT,
                allowNull: false,
                autoIncrement: true,
                primaryKey: true
            },
            'keyStr': sequelize_1.DataTypes.CHAR,
            'valueString': sequelize_1.DataTypes.CHAR,
            'valueInt': sequelize_1.DataTypes.BIGINT,
            'timestamp': sequelize_1.DataTypes.INTEGER,
            'createdAt': sequelize_1.DataTypes.DATE,
            'updatedAt': sequelize_1.DataTypes.DATE
        }, {
            'tableName': types_1.GLOBAL_PARAMS_TABLE,
            'timestamps': false
        });
    }
    getCurrValue(key) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.dbWriter.findOne({
                where: {
                    'keyStr': key
                }
            });
            return res;
        });
    }
    write(params) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.dbWriter.create(params);
            return res;
        });
    }
    writeAll(params) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.dbWriter.bulkCreate(params);
            return res;
        });
    }
}
exports.DbWriter2 = DbWriter2;
// test_Dbwriter()
