// 3. dbwriter dbwriter主要负责写数据库
import dayjs from 'dayjs';
import { Sequelize, DataTypes, Model, DateDataType, QueryTypes } from 'sequelize';
import { BinanceInfo, CHAIN, DbParams, DbParams2, GLOBAL_PARAMS_TABLE, SUFFIX, SumInfo, SYMBOL, symbolToAddrByChain } from './types';





export class DbWriter {
    dbWriter: any;
    chain: CHAIN;
    tableName: string;
    instance: any;
    globalPramasTable: string;


    constructor(connectionInfo: string, tableName: string, chain: CHAIN, globalParamsTable: string) {
        const instance = new Sequelize(connectionInfo, { "logging": true });
        this.globalPramasTable = globalParamsTable
        this.instance = instance
        this.chain = chain
        this.tableName = tableName
        this.dbWriter = instance.define('dbWriter', {
            'id': {
                type: DataTypes.BIGINT,
                allowNull: false,
                autoIncrement: true,
                primaryKey: true
            },
            'pool_address': DataTypes.CHAR,
            'token_address': DataTypes.CHAR,
            'chain_id': DataTypes.INTEGER,
            'balance': DataTypes.BIGINT,
            'timestamp': DataTypes.INTEGER,
            'createdAt': DataTypes.DATE,
            'updatedAt': DataTypes.DATE
        }, {
            'tableName': tableName,
            'timestamps': false
        })

    }

    async getLastReportSumBySymbol(symbol: SYMBOL): Promise<SumInfo> {
        let token_address: string = symbolToAddrByChain(symbol, this.chain)
        let suffix: SUFFIX = "_Last_Timestamp"
        let key: DbParams2["keyStr"] = `${this.chain}_${symbol}_${suffix}`
        let sql = `
        SELECT sum(balance) as balance, avg(timestamp) as timestamp
        FROM
            ${this.tableName}
        WHERE
            token_address = '${token_address}'
            and timestamp = IFNULL(
            (
                SELECT valueInt
                FROM
                    ${this.globalPramasTable}
                where
                    keyStr = '${key}'
                ORDER BY timestamp DESC
                LIMIT 1
            ),(
                SELECT TIMESTAMP
                FROM
                    ${this.tableName}
                where token_address = '${token_address}'
                ORDER BY TIMESTAMP 
                LIMIT 1
            ))
        order by timestamp
        limit 1
        `
        let res: Array<any> = await this.instance.query(sql, { type: QueryTypes.SELECT });
        let balance: number = Number(res[0].balance)
        let timestamp: number = Number(res[0].timestamp)
        if (timestamp < 1000) {
            console.error("get data from db failed timestamp: %d, balance %d", timestamp, balance)
            process.exit(1);
        }
        let sumInfo: SumInfo = { balance: balance, timestamp: timestamp, chain: this.chain, symbol: symbol }
        console.log("last type", typeof (balance))

        return sumInfo
    }
    async getFirstTodaySumBySymbol(symbol: SYMBOL): Promise<SumInfo> {
        let token_address: string = symbolToAddrByChain(symbol, this.chain)
        let sql = `
        SELECT sum(balance) as balance, avg(timestamp) as timestamp
        FROM
            ${this.tableName}
        WHERE
            token_address = '${token_address}'
            and timestamp = (
                SELECT TIMESTAMP
                FROM
                    ${this.tableName}
                where
                    Date(now()) = Date(FROM_UNIXTIME(timestamp))
                    and token_address = '${token_address}'
                ORDER BY TIMESTAMP
                LIMIT 1
            )
        limit 1
        `
        let res: Array<any> = await this.instance.query(sql, { type: QueryTypes.SELECT });
        let balance: number = Number(res[0].balance)
        let timestamp: number = Number(res[0].timestamp)
        if (timestamp < 1000) {
            console.error("get data from db failed timestamp: %d, balance %d", timestamp, balance)
            process.exit(1);
        }
        let sumInfo: SumInfo = { balance: balance, timestamp: timestamp, chain: this.chain, symbol: symbol }
        console.log("last type", typeof (balance))

        return sumInfo
    }
    async getCurrSumBySymbol(symbol: SYMBOL): Promise<SumInfo> {
        let token_address: string = symbolToAddrByChain(symbol, this.chain)
        let sql = `
        SELECT sum(balance) as balance, avg(timestamp) as timestamp
        FROM
            ${this.tableName}
        WHERE
            token_address = '${token_address}'
            and timestamp = (
                SELECT TIMESTAMP
                FROM
                    ${this.tableName}
                where 
                    token_address = '${token_address}'
                ORDER BY TIMESTAMP DESC
                LIMIT 1
            )
        limit 1
        `
        let res: Array<any> = await this.instance.query(sql, { type: QueryTypes.SELECT });
        // console.log(res)
        let balance: number = Number(res[0].balance)
        let timestamp: number = Number(res[0].timestamp)
        if (timestamp < 1000) {
            console.error("get data from db failed timestamp: %d, balance %d", timestamp, balance)
            process.exit(1);
        }
        let sumInfo: SumInfo = { balance: balance, timestamp: timestamp, chain: this.chain, symbol: symbol }
        console.log("last type", typeof (balance))

        return sumInfo
    }
    async write(params: DbParams) {
        let res = await this.dbWriter.create(params)
        return res
    }
    async writeAll(params: Array<DbParams>) {
        let res = await this.dbWriter.bulkCreate(params)
        return res
    }
}



export class DbWriter2 {
    dbWriter: any;

    constructor(connectionInfo: string) {
        const instance = new Sequelize(connectionInfo, { "logging": false });

        this.dbWriter = instance.define('dbWriter', {
            'id': {
                type: DataTypes.BIGINT,
                allowNull: false,
                autoIncrement: true,
                primaryKey: true
            },
            'keyStr': DataTypes.CHAR,
            'valueString': DataTypes.CHAR,
            'valueInt': DataTypes.BIGINT,
            'timestamp': DataTypes.INTEGER,
            'createdAt': DataTypes.DATE,
            'updatedAt': DataTypes.DATE
        }, {
            'tableName': GLOBAL_PARAMS_TABLE,
            'timestamps': false
        })
    }
    async getCurrValue(key: DbParams2["keyStr"]): Promise<DbParams2["valueString"] | DbParams2["valueInt"]> {
        let res = await this.dbWriter.findOne(
            {
                where: {
                    'keyStr': key
                }
            }
        )
        return res
    }
    async write(params: DbParams2) {
        let res = await this.dbWriter.create(params)
        return res
    }
    async writeAll(params: Array<DbParams2>) {
        let res = await this.dbWriter.bulkCreate(params)
        return res
    }

}
// test_Dbwriter()