"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const bnb_1 = require("./bnb");
const bsc_1 = require("./bsc");
const btc_1 = require("./btc");
const dbWriter_1 = require("./dbWriter");
const eth_1 = require("./eth");
const trx_1 = require("./trx");
const types_1 = require("./types");
require("log-timestamp");
function runEth(dbWriter) {
    return __awaiter(this, void 0, void 0, function* () {
        let resEth = yield (0, eth_1.getEth)();
        let resEthToDb = (0, types_1.parseBatch)(resEth);
        yield dbWriter.writeAll(resEthToDb);
    });
}
function runBtc(dbWriter) {
    return __awaiter(this, void 0, void 0, function* () {
        let resBtc = yield (0, btc_1.getBtc)();
        let resBtcToDb = (0, types_1.parseBatch)(resBtc);
        yield dbWriter.writeAll(resBtcToDb);
    });
}
function runTrx(dbWriter) {
    return __awaiter(this, void 0, void 0, function* () {
        let resTrx = yield (0, trx_1.getTrx)();
        let resTrxToDb = (0, types_1.parseBatch)(resTrx);
        yield dbWriter.writeAll(resTrxToDb);
    });
}
function runBnb(dbWriter) {
    return __awaiter(this, void 0, void 0, function* () {
        let resBnb = yield (0, bnb_1.getBnb)();
        let resBnbToDb = (0, types_1.parseBatch)(resBnb);
        yield dbWriter.writeAll(resBnbToDb);
    });
}
function runBsc(dbWriter) {
    return __awaiter(this, void 0, void 0, function* () {
        let resBsc = yield (0, bsc_1.getBsc)();
        let resBscToDb = (0, types_1.parseBatch)(resBsc);
        yield dbWriter.writeAll(resBscToDb);
    });
}
const SLEEP_TIME = 100000;
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        let trxDb = new dbWriter_1.DbWriter(types_1.connectionInfo, types_1.TRX_TABLE_NAME, "TRX", types_1.GLOBAL_PARAMS_TABLE);
        let ethDb = new dbWriter_1.DbWriter(types_1.connectionInfo, types_1.ETH_TABLE_NAME, "ETH", types_1.GLOBAL_PARAMS_TABLE);
        let btcDb = new dbWriter_1.DbWriter(types_1.connectionInfo, types_1.BTC_TABLE_NAME, "BTC", types_1.GLOBAL_PARAMS_TABLE);
        let bscDb = new dbWriter_1.DbWriter(types_1.connectionInfo, types_1.BSC_TABLE_NAME, "BSC", types_1.GLOBAL_PARAMS_TABLE);
        let bnbDb = new dbWriter_1.DbWriter(types_1.connectionInfo, types_1.BNB_TABLE_NAME, "BNB", types_1.GLOBAL_PARAMS_TABLE);
        while (true) {
            yield runEth(ethDb);
            yield runBsc(bscDb);
            yield runTrx(trxDb);
            yield runBnb(bnbDb);
            yield runBtc(btcDb);
            yield sleep(SLEEP_TIME);
        }
    });
}
main().catch((e) => {
    console.error(e);
    process.exit(1);
});
function sleep(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}
