import dayjs from 'dayjs';

import { getBnb } from './bnb';
import { getBsc } from './bsc';
import { getBtc } from './btc';
import { DbWriter } from './dbWriter';
import { getEth } from './eth';
import { getTrx } from './trx';
import { BNB_TABLE_NAME, BSC_TABLE_NAME, BTC_TABLE_NAME, connectionInfo, ETH_TABLE_NAME, GLOBAL_PARAMS_TABLE, parseBatch, TRX_TABLE_NAME } from './types';

require("log-timestamp")


async function runEth(dbWriter: DbWriter) {
    let resEth = await getEth();
    let resEthToDb = parseBatch(resEth)
    await dbWriter.writeAll(resEthToDb)
}
async function runBtc(dbWriter: DbWriter) {
    let resBtc = await getBtc();
    let resBtcToDb = parseBatch(resBtc)
    await dbWriter.writeAll(resBtcToDb)
}
async function runTrx(dbWriter: DbWriter) {
    let resTrx = await getTrx();
    let resTrxToDb = parseBatch(resTrx)
    await dbWriter.writeAll(resTrxToDb)
}
async function runBnb(dbWriter: DbWriter) {
    let resBnb = await getBnb();
    let resBnbToDb = parseBatch(resBnb)
    await dbWriter.writeAll(resBnbToDb)
}
async function runBsc(dbWriter: DbWriter) {
    let resBsc = await getBsc();
    let resBscToDb = parseBatch(resBsc)
    await dbWriter.writeAll(resBscToDb)
}
const SLEEP_TIME = 100000
async function main() {
    let trxDb = new DbWriter(connectionInfo, TRX_TABLE_NAME, "TRX", GLOBAL_PARAMS_TABLE)
    let ethDb = new DbWriter(connectionInfo, ETH_TABLE_NAME, "ETH", GLOBAL_PARAMS_TABLE)
    let btcDb = new DbWriter(connectionInfo, BTC_TABLE_NAME, "BTC", GLOBAL_PARAMS_TABLE)
    let bscDb = new DbWriter(connectionInfo, BSC_TABLE_NAME, "BSC", GLOBAL_PARAMS_TABLE)
    let bnbDb = new DbWriter(connectionInfo, BNB_TABLE_NAME, "BNB", GLOBAL_PARAMS_TABLE)
    while (true) {
        await runEth(ethDb)
        await runBsc(bscDb)
        await runTrx(trxDb)
        await runBnb(bnbDb)
        await runBtc(btcDb)
        await sleep(SLEEP_TIME)
    }
}

main().catch((e) => {
    console.error(e)
    process.exit(1)
})

function sleep(ms: number) {
    return new Promise(resolve => setTimeout(resolve, ms));
}
