"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Process = void 0;
const dayjs_1 = __importDefault(require("dayjs"));
const alarmer_1 = require("./alarmer");
const dbWriter_1 = require("./dbWriter");
const types_1 = require("./types");
require("log-timestamp");
const step = [0, -0.1, -0.2, -0.3, -0.4, -0.5, -0.6, -0.7, -0.8, -0.9, -1];
function getIndex(delta) {
    for (let i = 1; i < step.length; i++) {
        let cur = step[i];
        if (delta > cur)
            return i - 1;
    }
    return 0;
}
function judge(globalIndex, delta) {
    let curIndex = getIndex(delta);
    if (globalIndex < curIndex) {
        globalIndex = curIndex;
        return { globalIndex: globalIndex, shouldTrigger: true };
    }
    return { globalIndex: globalIndex, shouldTrigger: false };
}
function calSum(infos) {
    let res = 0;
    for (let i = 0; i < infos.length; i++) {
        res += infos[i].balance;
    }
    return res;
}
function getFirstTodayTimestamp(infos) {
    let res = 0;
    for (let i = 0; i < infos.length; i++) {
        if (res < infos[i].timestamp)
            res = infos[i].timestamp;
    }
    return res;
}
class Process {
    constructor(symbol, chains) {
        this.chains = [];
        this.dbWriters = [];
        this.globalIndex = 0;
        this.globalIndex2 = 0;
        this.firstTodayTimestamp = 0;
        this.symbol = symbol;
        this.chains = chains;
        this.info = { "symbol": this.symbol, "curr": [], "last": [], "firstToday": [] };
        this.dbGloabl = new dbWriter_1.DbWriter2(types_1.connectionInfo);
        for (let i = 0; i < chains.length; i++) {
            let tableName = (0, types_1.getTableNameByChain)(chains[i]);
            let dbWriter = new dbWriter_1.DbWriter(types_1.connectionInfo, tableName, chains[i], types_1.GLOBAL_PARAMS_TABLE);
            this.dbWriters.push(dbWriter);
        }
    }
    getData() {
        return __awaiter(this, void 0, void 0, function* () {
            this.info = { "symbol": this.symbol, "curr": [], "last": [], "firstToday": [] };
            for (let i = 0; i < this.dbWriters.length; i++) {
                let dbwriter = this.dbWriters[i];
                this.info.curr.push(yield dbwriter.getCurrSumBySymbol(this.symbol));
                this.info.last.push(yield dbwriter.getLastReportSumBySymbol(this.symbol));
                this.info.firstToday.push(yield dbwriter.getFirstTodaySumBySymbol(this.symbol));
            }
            let firstTodayTimestamp = getFirstTodayTimestamp(this.info.firstToday);
            if (this.firstTodayTimestamp < firstTodayTimestamp) {
                this.firstTodayTimestamp = firstTodayTimestamp;
                this.globalIndex = 0;
                console.log(this.symbol, " reset global index to 0");
            }
            return this.info;
        });
    }
    judgeToday() {
        let curr = calSum(this.info.curr);
        let firstToday = calSum(this.info.firstToday);
        let globalIndex = this.globalIndex;
        if (curr == 0 || firstToday == 0)
            return false;
        let delta = (curr - firstToday) / firstToday;
        let shouldTrigger = judge(globalIndex, delta).shouldTrigger;
        this.globalIndex = judge(globalIndex, delta).globalIndex;
        return shouldTrigger;
    }
    judgeLast() {
        let curr = calSum(this.info.curr);
        let last = calSum(this.info.last);
        let globalIndex2 = this.globalIndex2;
        if (curr == 0 || last == 0)
            return false;
        let delta = (curr - last) / last;
        let shouldTrigger = judge(globalIndex2, delta).shouldTrigger;
        this.globalIndex2 = judge(globalIndex2, delta).globalIndex;
        return shouldTrigger;
    }
    updateGlobalParamsTable(type) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            let globalParams = [];
            let suffix = "_Last_Timestamp";
            for (let i = 0; i < this.info.curr.length; i++) {
                let params = {
                    'keyStr': `${this.chains[i]}_${this.symbol}_${suffix}`,
                    'valueInt': (_a = this.info.curr[i].timestamp) !== null && _a !== void 0 ? _a : 0,
                    'valueString': type,
                    'timestamp': (0, dayjs_1.default)().unix()
                };
                globalParams.push(params);
            }
            yield this.dbGloabl.writeAll(globalParams);
        });
    }
    sendMessage(rule) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log(this.globalIndex);
            console.log(this.globalIndex2);
            console.log("triggerred");
            let curr = calSum(this.info.curr);
            let firstToday = calSum(this.info.firstToday);
            let last = calSum(this.info.last);
            let delta1 = (curr - firstToday) / firstToday;
            let delta2 = (curr - last) / last;
            let params = {
                "chain": this.chains,
                "token": this.symbol,
                "rule": rule,
                "datetime": (0, dayjs_1.default)().format("YYYY-MM-DD HH:mm:ssZ[Z]").toString(),
                "alarmInfo": {
                    curr: curr,
                    last: last,
                    firstToday: firstToday,
                    delta1: delta1,
                    delta2: delta2,
                    detail: JSON.stringify(this.info)
                }
            };
            yield (0, alarmer_1.send)(params);
        });
    }
    triggerToday() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.updateGlobalParamsTable("COMPARE_TO_FIRST_TODAY");
            yield this.sendMessage("COMPARE_TO_TODAY");
        });
    }
    triggerLast() {
        return __awaiter(this, void 0, void 0, function* () {
            this.globalIndex2 = 0;
            yield this.updateGlobalParamsTable("COMPARE_TO_LAST");
            yield this.sendMessage("COMPARE_TO_LAST");
        });
    }
}
exports.Process = Process;
function start(process) {
    return __awaiter(this, void 0, void 0, function* () {
        let info = yield process.getData();
        console.log(info);
        let shouldTriggerToday = process.judgeToday();
        let shouldTriggerLast = process.judgeLast();
        console.log("global index today", process.globalIndex);
        console.log("global index last", process.globalIndex2);
        console.log(shouldTriggerToday);
        console.log(shouldTriggerLast);
        if (shouldTriggerToday) {
            yield process.triggerToday();
        }
        if (shouldTriggerLast) {
            yield process.triggerLast();
        }
    });
}
function sleep(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        let bnbProcess = new Process("BNB", ["ETH", "BSC", "BNB"]);
        let ethProcess = new Process("ETH", ["ETH"]);
        let usdcProcess = new Process("USDC", ["TRX", "ETH"]);
        let usdtProcess = new Process("USDT", ["TRX", "ETH"]);
        let btcProcess = new Process("BTC", ["BTC"]);
        let busdProcess = new Process("BUSD", ["ETH"]);
        while (true) {
            yield sleep(10000);
            yield start(bnbProcess);
            yield start(ethProcess);
            yield start(usdcProcess);
            yield start(usdtProcess);
            yield start(btcProcess);
            yield start(busdProcess);
        }
    });
}
main().catch((e) => {
    console.error(e);
    process.exit(1);
});
// async function test() {
//     let process = new Process("BNB", ["ETH", "BSC", "BNB"]);
//     let info = await process.getData()
//     console.log(info)
//     let judge1 = process.judgeLast();
//     let judge2 = process.judgeToday();
//     await process.triggerToday();
//     // await process.sendMessage("COMPARE_TO_TODAY")
// }
// test()
