import dayjs from "dayjs";
import { send } from "./alarmer";
import { DbWriter, DbWriter2 } from "./dbWriter";
import { sendMessage } from "./helper";
import { AlamerParams, CHAIN, connectionInfo, DataInfo, DbParams2, getTableNameByChain, GLOBAL_PARAMS_TABLE, RULE, SUFFIX, SumInfo, SYMBOL } from "./types";
require("log-timestamp")



const step = [0, -0.1, -0.2, -0.3, -0.4, -0.5, -0.6, -0.7, -0.8, -0.9, -1]
function getIndex(delta: number) {
    for (let i = 1; i < step.length; i++) {
        let cur = step[i]
        if (delta > cur) return i - 1
    }
    return 0
}
function judge(globalIndex: number, delta: number) {
    let curIndex = getIndex(delta)
    if (globalIndex < curIndex) {
        globalIndex = curIndex
        return { globalIndex: globalIndex, shouldTrigger: true }
    }
    return { globalIndex: globalIndex, shouldTrigger: false }
}
function calSum(infos: SumInfo[]): number {
    let res = 0
    for (let i = 0; i < infos.length; i++) {
        res += infos[i].balance
    }
    return res
}
function getFirstTodayTimestamp(infos: SumInfo[]): number {
    let res = 0

    for (let i = 0; i < infos.length; i++) {
        if (res < infos[i].timestamp) res = infos[i].timestamp
    }
    return res
}
export class Process {
    symbol: SYMBOL;
    chains: CHAIN[] = [];
    dbWriters: DbWriter[] = [];
    alamer: any;
    info: DataInfo;
    globalIndex = 0;
    globalIndex2 = 0;
    dbGloabl: DbWriter2;
    firstTodayTimestamp: number = 0;
    constructor(symbol: SYMBOL, chains: CHAIN[]) {
        this.symbol = symbol
        this.chains = chains
        this.info = { "symbol": this.symbol, "curr": [], "last": [], "firstToday": [] }
        this.dbGloabl = new DbWriter2(connectionInfo)
        for (let i = 0; i < chains.length; i++) {
            let tableName = getTableNameByChain(chains[i])
            let dbWriter = new DbWriter(connectionInfo, tableName, chains[i], GLOBAL_PARAMS_TABLE)
            this.dbWriters.push(dbWriter)
        }

    }
    async getData() {
        this.info = { "symbol": this.symbol, "curr": [], "last": [], "firstToday": [] }
        for (let i = 0; i < this.dbWriters.length; i++) {
            let dbwriter = this.dbWriters[i]
            this.info.curr.push(await dbwriter.getCurrSumBySymbol(this.symbol))
            this.info.last.push(await dbwriter.getLastReportSumBySymbol(this.symbol))
            this.info.firstToday.push(await dbwriter.getFirstTodaySumBySymbol(this.symbol))
        }
        let firstTodayTimestamp = getFirstTodayTimestamp(this.info.firstToday)
        if (this.firstTodayTimestamp < firstTodayTimestamp) {
            this.firstTodayTimestamp = firstTodayTimestamp
            this.globalIndex = 0
            console.log(this.symbol, " reset global index to 0")
        }
        return this.info
    }
    judgeToday(): boolean {
        let curr = calSum(this.info.curr)
        let firstToday = calSum(this.info.firstToday)
        let globalIndex = this.globalIndex
        if (curr == 0 || firstToday == 0) return false;
        let delta = (curr - firstToday) / firstToday

        let shouldTrigger = judge(globalIndex, delta).shouldTrigger
        this.globalIndex = judge(globalIndex, delta).globalIndex
        return shouldTrigger
    }
    judgeLast(): boolean {
        let curr = calSum(this.info.curr)
        let last = calSum(this.info.last)
        let globalIndex2 = this.globalIndex2
        if (curr == 0 || last == 0) return false;
        let delta = (curr - last) / last

        let shouldTrigger = judge(globalIndex2, delta).shouldTrigger
        this.globalIndex2 = judge(globalIndex2, delta).globalIndex
        return shouldTrigger
    }
    async updateGlobalParamsTable(type: string) {
        let globalParams: DbParams2[] = []
        let suffix: SUFFIX = "_Last_Timestamp"

        for (let i = 0; i < this.info.curr.length; i++) {
            let params: DbParams2 = {
                'keyStr': `${this.chains[i]}_${this.symbol}_${suffix}`,
                'valueInt': this.info.curr[i].timestamp ?? 0,
                'valueString': type,
                'timestamp': dayjs().unix()
            }
            globalParams.push(params)
        }
        await this.dbGloabl.writeAll(globalParams)
    }
    async sendMessage(rule: RULE) {
        console.log(this.globalIndex)
        console.log(this.globalIndex2)
        console.log("triggerred");
        let curr = calSum(this.info.curr)
        let firstToday = calSum(this.info.firstToday)
        let last = calSum(this.info.last)
        let delta1 = (curr - firstToday) / firstToday
        let delta2 = (curr - last) / last
        let params: AlamerParams = {
            "chain": this.chains,
            "token": this.symbol,
            "rule": rule,
            "datetime": dayjs().format("YYYY-MM-DD HH:mm:ssZ[Z]").toString(),
            "alarmInfo": {
                curr: curr,
                last: last,
                firstToday: firstToday,
                delta1: delta1,
                delta2: delta2,
                detail: JSON.stringify(this.info)
            }

        }
        await send(params)
    }
    async triggerToday() {
        await this.updateGlobalParamsTable("COMPARE_TO_FIRST_TODAY")
        await this.sendMessage("COMPARE_TO_TODAY")
    }
    async triggerLast() {
        this.globalIndex2 = 0
        await this.updateGlobalParamsTable("COMPARE_TO_LAST")
        await this.sendMessage("COMPARE_TO_LAST")
    }

}

async function start(process: Process) {
    let info = await process.getData()
    console.log(info)
    let shouldTriggerToday = process.judgeToday();
    let shouldTriggerLast = process.judgeLast();
    console.log("global index today", process.globalIndex)
    console.log("global index last", process.globalIndex2)
    console.log(shouldTriggerToday)
    console.log(shouldTriggerLast)

    if (shouldTriggerToday) {
        await process.triggerToday()
    }
    if (shouldTriggerLast) {
        await process.triggerLast()
    }

}
function sleep(ms: number) {
    return new Promise(resolve => setTimeout(resolve, ms));
}
async function main() {

    let bnbProcess: Process = new Process("BNB", ["ETH", "BSC", "BNB"]);
    let ethProcess: Process = new Process("ETH", ["ETH"])
    let usdcProcess: Process = new Process("USDC", ["TRX", "ETH"]);
    let usdtProcess: Process = new Process("USDT", ["TRX", "ETH"])
    let btcProcess: Process = new Process("BTC", ["BTC"]);
    let busdProcess: Process = new Process("BUSD", ["ETH"]);

    while (true) {
        await sleep(10000)
        await start(bnbProcess)
        await start(ethProcess)
        await start(usdcProcess)
        await start(usdtProcess)
        await start(btcProcess)
        await start(busdProcess)
    }


}

main().catch((e) => {
    console.error(e)
    process.exit(1)
})

// async function test() {
//     let process = new Process("BNB", ["ETH", "BSC", "BNB"]);
//     let info = await process.getData()
//     console.log(info)
//     let judge1 = process.judgeLast();
//     let judge2 = process.judgeToday();
//     await process.triggerToday();
//     // await process.sendMessage("COMPARE_TO_TODAY")
// }
// test()