import dayjs from "dayjs"
import { BigNumber } from "ethers"
// export const connectionInfo: string = 'mysql://root:admin123@localhost:3306/monitor';
// export const connectionInfo: string = 'mysql://root:admin123@localhost:3306/binance';
export const connectionInfo: string = 'mysql://root:123456@47.74.235.176:3206/binance';
export const GLOBAL_PARAMS_TABLE: string = 'global_params'
export type BinanceInfo = {
    "account": string,
    "balance": string,
    "decimals": number,
    "symbol": SYMBOL,
    "chain": CHAIN
}
export type SumInfo = {
    "balance": number,
    "timestamp": number,
    "chain": CHAIN,
    "symbol": SYMBOL
}
export type DataInfo = {
    "symbol": SYMBOL,
    "curr": SumInfo[],
    "last": SumInfo[],
    "firstToday": SumInfo[]
}
export type DbParams = {
    "pool_address": string,
    "token_address": string,
    "balance": number,
    "chain_id": CHAIN_ID,
    "timestamp": number
}
export type DbParams2 = {
    "keyStr": `${CHAIN}_${SYMBOL}_${SUFFIX}`,
    "valueString": string,
    "valueInt": number,
    "timestamp": number
}
export type AlarmInfo = {
    "curr": number,
    "last": number,
    "firstToday": number,
    "delta1": number,
    "delta2": number,
    "detail": string
}
export type AlamerParams = {
    "token": SYMBOL,
    "chain": CHAIN[],
    "rule": RULE,
    "alarmInfo": AlarmInfo,
    "datetime": string
}
export type RULE = "COMPARE_TO_TODAY" | "COMPARE_TO_LAST"
export type CHAIN = "BNB" | "BSC" | "BTC" | "TRX" | "ETH"
export type SYMBOL = "BNB" | "ETH" | "BTC" | "USDC" | "USDT" | "BUSD"
export type CHAIN_ID = 1 | 56 | -1 | -2 | -3

export const ETH_TABLE_NAME = "record_balance_ethereum_binance_wallet_balance"
export const BTC_TABLE_NAME = "record_balance_bitcoin_binance_wallet_balance"
export const TRX_TABLE_NAME = "record_balance_trx_binance_wallet_balance"
export const BNB_TABLE_NAME = "record_balance_bnb_binance_wallet_balance"
export const BSC_TABLE_NAME = "record_balance_bsc_binance_wallet_balance"


export type SUFFIX = "_Last_Timestamp" | "_Toady_Timestamp"

export const BTC = "BTC"
export const BNB_BNB = "BNB_BNB"
export const BSC_BNB = "0xbb4CdB9CBd36B01bD1cBaEBF2De08d9173bc095c"
export const TRX_USDT = "TR7NHqjeKQxGTCi8q8ZY4pL8otSzgjLj6t"
export const TRX_USDC = "TEkxiTehnzSmSe2XqrBj4w32RUN966rdz8"
export const ETH_USDT = "0xdAC17F958D2ee523a2206206994597C13D831ec7"
export const ETH_USDC = "0xA0b86991c6218b36c1d19D4a2e9Eb0cE3606eB48"
export const ETH_BUSD = "0x4Fabb145d64652a948d72533023f6E7A623C7C53"
export const ETH_BNB = "0xB8c77482e45F1F44dE1745F52C74426C631bDD52"
export const ETH_ETH = "0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE"



export function parseBinanceInfoToDb(info: BinanceInfo): DbParams {
    let precision = BigNumber.from(10).pow(info.decimals)
    let db: DbParams = {
        "pool_address": info.account,
        "token_address": symbolToAddrByChain(info.symbol, info.chain),
        "balance": BigNumber.from(info.balance).div(precision).toNumber(),
        "chain_id": chainToChainId(info.chain),
        "timestamp": dayjs().unix()
    }
    return db
}
export function parseBatch(infos: BinanceInfo[]): DbParams[] {
    let res: DbParams[] = []
    for (let i = 0; i < infos.length; i++) {
        res.push(parseBinanceInfoToDb(infos[i]))
    }
    return res
}
export function chainToChainId(chain: CHAIN): CHAIN_ID {
    switch (chain) {
        case "BSC":
            return 56
        case "ETH":
            return 1
        case "BTC":
            return -1
        case "TRX":
            return -2
        case "BNB":
            return -3
    }

}
export function symbolToAddrByChain(symbol: string, chain: CHAIN): string {
    switch (true) {
        case ((chain === "BTC")): return BTC;
        case ((chain === "BNB")): return BNB_BNB;
        case ((chain === "BSC")): return BSC_BNB;
        case ((chain === "TRX") && (symbol === "USDT")): return TRX_USDT;
        case ((chain === "TRX") && (symbol === "USDC")): return TRX_USDC;
        case ((chain === "ETH") && (symbol === "USDT")): return ETH_USDT;
        case ((chain === "ETH") && (symbol === "USDC")): return ETH_USDC;
        case ((chain === "ETH") && (symbol === "ETH")): return ETH_ETH;
        case ((chain === "ETH") && (symbol === "BUSD")): return ETH_BUSD;
        case ((chain === "ETH") && (symbol === "BNB")): return ETH_BNB;

    }
    return ""
}
export function getTableNameByChain(chain: CHAIN): string {
    switch (true) {
        case (chain === "BTC"): return BTC_TABLE_NAME;
        case (chain === "ETH"): return ETH_TABLE_NAME;
        case (chain === "TRX"): return TRX_TABLE_NAME;
        case (chain === "BSC"): return BSC_TABLE_NAME;
        case (chain === "BNB"): return BNB_TABLE_NAME;
    }
    return ""
}